@@ -6,7 +6,7 @@ package NexTrieve::Targz;
 
 use strict;
 @NexTrieve::Targz::ISA = qw(NexTrieve);
-$NexTrieve::Targz::VERSION = '0.30';
+$NexTrieve::Targz::VERSION = '0.31';
 
 # Make sure we use the modules that we need here
 
@@ -24,53 +24,12 @@ my $newmessagedelimiter = '^From ';
 
 #------------------------------------------------------------------------
 
-# The following methods are class methods
-
-#------------------------------------------------------------------------
-
-# if called as a class method
-#  IN: 1 Targz work directory
-# OUT: 1 exit status of "rm"
-
-# if called as an object method
-# OUT: 1 exit status of "rm"
-
-sub clean {
-
-# Initialize the directory
-# If called as an object method
-#  Obtain the work directory from the object
-# Else (subroutine or class method call)
-#  Remove class if it looks like a class
-#  Obtain the directory from specified parameters
-
-  my $work;
-  if (ref($_[0])) {
-    $work = shift->work;
-  } else {
-    shift if $_[0] =~ m#::#;
-    $work = shift;
-  }
-
-# Clean out directory name if attempt at bad stuff
-# Return now if we don't have a directory
-# Return now if it is not a directory
-# Remove everything in the directory and return the exit code
-
-  $work =~ s#^(?:/|/\w+)$##;
-  return unless length($work);
-  return unless -d $work;
-  return system( "rm -rf $work/*" );
-} #clean
-
-#------------------------------------------------------------------------
-
 # The following methods return objects
 
 #------------------------------------------------------------------------
 
 #  IN: 1 NexTrieve object
-#      2 directory to work on
+#      2 directory to work on (assume current directory)
 #      3 reference to parameter hash to be passed to object (optional)
 # OUT: 1 instantiated NexTrieve::Targz object
 
@@ -86,11 +45,13 @@ sub _new {
 # Make sure it's absolute
 # Obtain the name of the directory to use
 # Make sure it's absolute
+# Remove trailing slash and optional dot if there is one
 
   my $work = $self->Tmp.'/Targz';
   $work = cwd."/$work" unless $work =~ s#^\s*(?=/)##;
-  my $directory = shift;
+  my $directory = shift || '';
   $directory = cwd."/$directory" unless $directory =~ s#^\s*(?=/)##;
+  $directory =~ s#/\.?$##;
 
 # For all of the (sub)directories that should exist
 #  Reloop if it exists
@@ -103,13 +64,11 @@ sub _new {
 
 # Set the real work directory to work on
 # Set the directory to work on
-# Make sure we clean up by default
 # Process any parameter hash if specified
 # Return the object
 
   $self->{$class.'::work'} = "$work/$$";
   $self->{$class.'::directory'} = $directory;
-  $self->auto_clean( 1 );
   $self->Set( @_ ) if @_;
   return $self;
 } #_new
@@ -196,7 +155,7 @@ sub add_file {
 
 # If failed to move to right directory
 #  Add error and return
-# Obtain the base name of the tar-files to work with
+# Obtain the base name of the tarfiles to work with
 
   unless (chdir( $directory )) {
     $self->_add_error( "Cannot chdir to $directory" );
@@ -234,7 +193,7 @@ sub add_file {
       push( @unlink,$filename ) if $rm_original;
       my $time = _epoch( $handle );
       unless ($time) {
-        $self->_add_error( "Could not determine originating time of $_" );
+#        $self->_add_error( "Could not determine originating time of $_" );
         next;
       }
       next if _handle_file( $self,$directory,$name,$filename,$time,\%ordinal );
@@ -280,7 +239,7 @@ sub add_mbox {
 
 # If failed to move to right directory
 #  Add error and return
-# Obtain the base name of the tar-files to work with
+# Obtain the base name of the tarfiles to work with
 
   unless (chdir( $directory )) {
     $self->_add_error( "Cannot chdir to $directory" );
@@ -330,7 +289,6 @@ sub add_mbox {
           $linenumber++;
           $message .= $line;
         }
-        my $message_handle = bless \$message,'NexTrieve::handle';
 
 #   Attempt to obtain a time value for this message
 #   If we don't have a time
@@ -338,9 +296,9 @@ sub add_mbox {
 #   Handle all the stuff needed for this file, reloop if ok
 #   Return now indicating error
 
-        my $time = _epoch( $message_handle );
+        my $time = _epoch( \$message );
         unless ($time) {
-          $self->_add_error( "Could not determine originating time of message at line $start in $mbox" );
+#          $self->_add_error( "Could not determine originating time of message at line $start in $mbox" );
           next;
         }
         next if _handle_file($self,$directory,$name,$message,$time,\%ordinal);
@@ -362,40 +320,245 @@ sub add_mbox {
 
 #------------------------------------------------------------------------
 
-#  IN: 1 new value for "auto_clean" action
-# OUT: 1 current/old value for "auto_clean" action
+#  IN: 1 Net::NNTP object for fetching news messages from
+# OUT: 1 whether successful
 
-sub auto_clean { shift->_class_variable( 'auto_clean',@_ ) } #auto_clean
+sub add_news {
+
+# Obtain the object
+# Obtain the Net::NNTP object
+# Obtain the class
+# Obtain the RFC822 object if any
+# Obtain the Docseq object if any
+
+  my $self = shift;
+  my $nntp = shift;
+  my $class = ref($self);
+  my $rfc822 = $self->{$class.'::rfc822'} || '';
+  my $docseq = $self->{$class.'::docseq'} || '';
+
+# Obtain the original directory
+# Obtain the directory to work in
+# If failed to move to right directory
+#  Add error and return
+# Obtain the base name of the tarfiles to work with
+
+  my $cwd = cwd;
+  my $directory = $self->directory;
+  unless (chdir( $directory )) {
+    $self->_add_error( "Cannot chdir to $directory" );
+    return;
+  }
+  (my $newsgroup = $directory) =~ s#^.*/##;
+
+# Initialize the hash with ordinals
+# Obtain the ordinal number of first and last message to fetch
+
+  my %ordinal;
+  my $messages = 0;
+  my ($first,$last) = _resync_news( $self,$nntp,$newsgroup );
+
+# For all of the messages
+#  Obtain the message
+#  Reloop if nothing there (probably removed from server)
+
+  for (my $i = $first; $i <= $last; $i++) {
+    my $message = $nntp->article( $i );
+    next unless $message;
+
+#   Attempt to obtain a time value for this message
+#   If we don't have a time
+#    Add error message and reloop
+
+    my $time = _epoch( $message );
+    unless ($time) {
+#      $self->_add_error( "Could not determine originating time of message $i in $newsgroup" );
+      next;
+    }
+
+#   If handling all the stuff needed for this file was ok
+#    Increment number of messages
+#    And reloop
+#   Return now indicating error
+
+    if (_handle_file($self,$directory,$newsgroup,$message,$time,\%ordinal)) {
+      $messages++;
+      next;
+    }
+    return '';
+  }
+
+# Finish up the adding process
+# Restore the original directory
+# Return indicating success
+
+  _finish_add( $self,$directory,$newsgroup,\%ordinal,$rfc822,$docseq );
+  chdir( $cwd );
+  return $messages;
+} #add_news
+
+#------------------------------------------------------------------------
+
+# OUT: 1 exit status of "rm"
+
+sub clean {
+
+# Obtain the work directory from the object
+# Clean out directory name if attempt at bad stuff
+
+  my $work = shift->work;
+  $work =~ s#^(?:/|/\w+)$##;
+
+# Return now if we don't have a directory
+# Return now if it is not a directory
+# Remove everything in the directory and return the exit code
+
+  return unless length($work);
+  return unless -d $work;
+  return system( "rm -rf $work/*" );
+} #clean
+
+#------------------------------------------------------------------------
+
+#  IN: 1 regular expression limiting the dates from which to count files
+#      2 reference to a hash: key = date, value = [last mod,count]
+# OUT: 1 total number of files
+
+sub count {
+
+# Obtain the object
+# Obtain the constraint
+# Obtain the external hash reference
+# Return with the number of entries if there is no external hash reference
+
+  my $self = shift;
+  my $constraint = shift || '';
+  my $hash = shift || '';
+  return scalar @{$self->ids( $constraint )} unless $hash;
+
+# Obtain the directory
+# Obtain the name
+# Add the RFC subdirectory
+# Initialize the number of messages
+
+  my $directory = $self->directory;
+  (my $name = $directory) =~ s#^.*/##;
+  $directory .= '/rfc';
+  my $messages = 0;
+
+# For all of the datestamps according to this constraint
+#  Reloop if there is no tarfile for it
+#  Obtain the last modified info from the tarfile
+#  If there is an entry in the hash already
+#   If the tarfile didn't change
+#    Increment number of messages and reloop
+
+  foreach my $datestamp (@{$self->datestamps( $constraint )}) {
+    next unless -e "$directory/$datestamp.$name.tar.gz";
+    my $thistime = (stat(_))[9];
+    if (exists( $hash->{$datestamp} )) {
+      if ($thistime == $hash->{$datestamp}->[0]) {
+        $messages += $hash->{$datestamp}->[1];
+        next;
+      }
+    }
+
+#  Obtain the number of messages
+#  Set the entry in the hash
+#  Increment number of messages
+
+    my $thismessages = @{$self->ids( $datestamp )};
+    $hash->{$datestamp} = [$thistime,$thismessages];
+    $messages += $thismessages;
+  }
+
+# Return the number of messages
+
+  return $messages;
+} #count
 
 #------------------------------------------------------------------------
 
 #  IN: 1 regular expression limiting the dates from which to count files
 # OUT: 1 total number of files
 
-sub count {
+sub count_storable {
 
 # Obtain the object
-# Obtain the list of files
-# Return just the number of entries
+# If we didn't try to check for Storable yet
+#  Attempt to load Storable
+#  Set flag if still not successful
+# Return result of ordinary count if Storable is not available
 
   my $self = shift;
-  my $list = $self->ids( @_ );
-  return scalar @{$list};
-} #count
+  unless (defined($Storable::VERSION)) {
+    eval 'use Storable ()';
+    $Storable::VERSION ||= '';
+  }
+  return $self->count( shift ) unless $Storable::VERSION;
+
+# Create the filename we need to look for
+# Initialize the hash ref
+# If the frozen version of the hash already exists
+#  Make sure we slurp always
+#  If we could open a pipe to that file
+#   Obtain the hash ref from it
+#   Close the pipe
+
+  my $filename = $self->directory.'/count.gz';
+  my $hash;
+  if (-e $filename) {
+    local $/ = undef;
+    if (my $handle = $self->openfile( "gunzip --stdout $filename |" )) {
+      $hash = Storable::fd_retrieve( $handle );
+      close( $handle );
+    }
+  }
+
+# Make sure that we have a have ref if something went wrong
+# Perform the normal count, but with a hash ref
+
+  $hash ||= {};
+  my $count = $self->count( shift,$hash );
+
+# Make sure that the file exists with the right ownerships
+# If we can save the result, we can open a pipe
+#  Write the hash to the file using a pipe
+#  Close the pipe
+# Return what we found
+
+  $self->openfile( $filename,'>' ) unless -e $filename;
+  if (my $handle = $self->openfile( "|gzip --stdout >$filename" )) {
+    Storable::store_fd( $hash,$handle );
+    close( $handle );
+  }
+  return $count;
+} #count_storable
 
 #------------------------------------------------------------------------
 
+#  IN: 1 regular expression limiting the dates to return
 # OUT: 1 reference to datestamps of dates in this targz
 
 sub datestamps {
 
 # Obtain the object
-# Obtain the directory name
-# Obtain the list of datestamps and return a reference to it
+# Obtain the constraint (if any)
+# Return now with that datestamp only if it is a simple datestamp constraint
 
   my $self = shift;
+  my $constraint = shift || '';
+  return [$constraint] if $constraint and $constraint =~ m#^\d{8}$#;
+
+# Obtain the directory name
+# Obtain the list of datestamps
+# Just keep the ones we really want if there is a constraint
+# Return a reference to the list
+
   my $directory = $self->directory;
-  return [map {m#(\d{8}).*?\.tar\.gz$#; $1} <$directory/rfc/*.tar.gz>];
+  my @date = map {m#(\d{8}).*?\.tar\.gz$#; $1} <$directory/rfc/*.tar.gz>;
+  @date = grep( m#$constraint#,@date ) if $constraint;
+  return \@date;
 } #datestamps
 
 #------------------------------------------------------------------------
@@ -439,30 +602,12 @@ sub ids {
 
 # Obtain the object
 # Obtain the constraint
+# Fetch all dates
+# Return now if nothing to do
 
   my $self = shift;
   my $constraint = shift || '';
-
-# Initialize the dates
-# If there is a constraint
-#  If it is a simple datestamp
-#   Set that as the date
-#  Else (not a simple datestamp, a real constraint)
-#   Perform a grep all dates
-# Else (n constraint)
-#  Fetch all dates
-# Return now if nothing left
-
-  my @date;
-  if ($constraint) {
-    if ($constraint =~ m#^\d{8}#) {
-      $date[0] = $constraint;
-    } else {
-      @date = grep( m#$constraint#,@{$self->datestamps} );
-    }
-  } else {
-    @date = @{$self->datestamps};
-  }
+  my @date = @{$self->datestamps( $constraint )};
   return unless @date;
 
 # Obtain the directory
@@ -495,6 +640,28 @@ sub ids {
 
 #------------------------------------------------------------------------
 
+# OUT: 1 name of targz
+
+sub name {
+
+# Obtain the directory
+# Obtain the name part
+# And return that
+
+  my $directory = shift->directory;
+  (my $name = $directory) =~ s#^.*/##;
+  return $name;
+} #name
+
+#------------------------------------------------------------------------
+
+#  IN: 1 new value for "no_auto_clean" action
+# OUT: 1 current/old value for "no_auto_clean" action
+
+sub no_auto_clean { shift->_class_variable('no_auto_clean',@_) } #no_auto_clean
+
+#------------------------------------------------------------------------
+
 #  IN: 1 new setting of "rm_original" flag
 # OUT: 1 old/current setting of "rm_original" flag
 
@@ -502,6 +669,37 @@ sub rm_original { shift->_class_variable( 'rm_original',@_ ) } #rm_original
 
 #------------------------------------------------------------------------
 
+#  IN: 1 datestamp of which to obtain tarfile name
+#      2 type of information (default: 'rfc')
+# OUT: 1 name of tarfile
+
+sub tarfile {
+
+# Obtain the object
+# Obtain the datestamp
+# Return now if nothing to return
+
+  my $self = shift;
+  my $datestamp = shift || '';
+  return '' unless $datestamp;
+
+# Obtain the type of information
+# Remove prefix period if any
+
+  my $type = shift || 'rfc';
+  $type =~ s#^\.##;
+
+# Obtain the directory
+# Obtain the name
+# Return the name of the tarfile
+
+  my $directory = $self->directory;
+  (my $name = $directory) =~ s#^.*/##;
+  return "$directory/$type/$datestamp.$name.tar.gz";
+} #tarfile
+
+#------------------------------------------------------------------------
+
 #  IN: 1 RFC822 object to create XML with (default: from object)
 #      2 Docseq object to create document sequence with (default: from object)
 # OUT: 1 number of files processed
@@ -611,7 +809,7 @@ sub xml {
 EOD
 
 # Obtain the directory to work in
-# Obtain the base name of the tar-files to work with
+# Obtain the base name of the tarfiles to work with
 # Add the XML subdirectory
 # If directory doesn't exist
 #  Add error and return
@@ -674,7 +872,7 @@ sub DESTROY {
 # Remove the directory if so specified
 
   my $self = shift;
-  $self->clean if $self->auto_clean;
+  $self->clean unless $self->no_auto_clean;
 } #DESTROY
 
 #------------------------------------------------------------------------
@@ -733,23 +931,52 @@ sub _datestamp {
 
 #------------------------------------------------------------------------
 
-#  IN: 1 file handle (closed upon exit in scalar context)
+#  IN: 1 file handle (closed upon exit in scalar context) or ref to message
 # OUT: 1 timestamp
 #      2 all headers read until date found
 
 sub _epoch {
 
-# Obtain the handle
+# Obtain the reference
 # Initialize the line
 # Initialize the time
-# Save the internal handle flag
-# Set the routine to fetch the next line
+# Initialize the close flag
+# Initialize the handle
+# Initialize the loop
 
-  my $handle = shift;
+  my $ref = shift;
   my $line = '';
   my $time = '';
-  my $internal = ref($handle) eq 'NexTrieve::handle';
-  my $next = $internal ?  sub {$handle->next} : sub {<$handle>};
+  my $close = '';
+  my $handle;
+  my $next;
+
+# If a message or header was passed
+#  Create the handle
+#  Set the routine to fetch the next line
+
+  if (ref($ref) eq 'SCALAR') {
+    $handle = bless $ref,'NexTrieve::handle';
+    $next = sub {$handle->next};
+
+# Elseif a list reference
+#  Create the index
+#  Set the routine to fetch the next line
+
+  } elsif (ref($ref) eq 'ARRAY') {
+    $handle = 0;
+    $next = sub{$handle < @{$ref} ? $ref->[$handle++] : ''};
+
+# Else (assume filehandle to read from)
+#  Set the handle
+#  Set the routine to fetch the next line
+#  Set the flag to close the handle
+
+  } else {
+    $handle = $ref;
+    $next = sub{<$handle>};
+    $close = 1;
+  }
 
 # Make sure we have a local version of $_ (strange things happen otherwise)
 # While there are lines to be read (obtaining it in $_)
@@ -765,43 +992,48 @@ sub _epoch {
 
 #  Else (new line)
 #   If we have a line, making sure it is lowercase on the fly
-#    If it is something that could be a date
-#     Outloop if we got back something valid
+#    If it is a line we think can have a valid date
+#     If there is something that could be a date
+#      Outloop if we got back something valid
 #   Start a new line
 
     } else {
       if ($line = lc($line)) {
-        if ($line =~ m#^from [\w\-\.=@]+\s*(.+?)\s*$# or
-            $line =~ m#^date:\s*(.+?)\s*$# or
-            $line =~ m#[^;]*;(.+)#
-           ) {
-          last if $time = _str2time( $1 );
+        if ($line =~ m#^(?:date:|from |nntp-posting-date:|received:|x-trace:)#) {
+          if ($line =~ m#(\d+\s+\w+\s+\d+\s+\d+:\d+:\d+(?:\s+(?:[+-]?\d+|[a-zA-Z]+))?)#) {
+            last if $time = _str2time( $1 );
+          }
         }
       }
       $line = $_;
     }
   }
 
-# Close the file (we can't do anything with it anymore)
+# Close the handle if so requested
 # Return whatever we got
 
-  close( $handle ) unless $internal;
+  close( $handle ) if $close;
   return $time;
 } #_epoch
 
 #------------------------------------------------------------------------
 
 #  IN: 1 object
-#      2 tarfile to extract file to current directory to
+#      2 tarfile to extract file
+#      3 directory to extract to (default: current directory)
 # OUT: 1 reference of list of names of files
 
 sub _extract_tarfile {
 
 # Obtain the object
 # Obtain the name of the tarfile
+# Obtain the directory to extract to
+# Turn it into a parameter if specified
 
   my $self = shift;
   my $tarfile = shift;
+  my $todir = shift || '';
+  $todir = " --directory=$todir" if $todir;
 
 # If we want a list of filenames returned
 #  If the tarfile exists
@@ -813,8 +1045,8 @@ sub _extract_tarfile {
 
   if (defined(wantarray)) {
     if (-e $tarfile and -s _) {
-      if (my $handle =
-       $self->openfile( "tar --extract --gunzip --verbose --file=$tarfile|" )) {
+      if (my $handle = $self->openfile(
+       "tar --extract --gunzip --verbose$todir --file=$tarfile|" )) {
         chomp( my @file = <$handle> ); @file = sort {$a <=> $b} @file;
         close( $handle );
         return \@file;
@@ -829,7 +1061,7 @@ sub _extract_tarfile {
 #   Add error
 
   if (-e $tarfile and -s _) {
-    my $command = "tar --extract --gunzip --file=$tarfile";
+    my $command = "tar --extract --gunzip$todir --file=$tarfile";
     if (my $exit = system( $command )) {
       $self->_add_error( "Command '$command' returned status $exit" );
     }
@@ -945,18 +1177,29 @@ sub _handle_file {
 
 # Calculate the ID to be used for this file
 # Save that in the file list
-# If we got a direct message
-#  Just splat the file where it's supposed to be and return if successful
-#  Add error
-# Else (we've got to copy the file because it exists out there already)
-#  Return now indicating success if copy of file to temp directory successful
+# If we got a reference to a list
+#  Splat the joined contents where it's supposed to be and return if successful
 #  Add error
 
   my $id = @{$ordinal->{$date}} ? $ordinal->{$date}->[-1] + 1 : $date + 1;
   push( @{$ordinal->{$date}},$id );
-  if ($file =~ m#\n#s) {
+  if (ref($file) eq 'ARRAY') {
+    return 1 if $self->splat(
+     $self->openfile( "$temp/$id",'>' ),join( '',@{$file} ) );
+    $self->_add_error( "Error saving message to '$temp/$id': $!" );
+
+# Elseif we got a direct message
+#  Just splat the file where it's supposed to be and return if successful
+#  Add error
+
+  } elsif ($file =~ m#\n#s) {
     return 1 if $self->splat( $self->openfile( "$temp/$id",'>' ),$file );
     $self->_add_error( "Error saving message to '$temp/$id': $!" );
+
+# Else (we've got to copy the file because it exists out there already)
+#  Return now indicating success if copy of file to temp directory successful
+#  Add error
+
   } else {
     return 1 if copy( $file,"$temp/$id" );
     $self->_add_error( "Error copying '$file' to '$temp/$id': $!" );
@@ -999,6 +1242,205 @@ sub _list_tarfile {
 
 #------------------------------------------------------------------------
 
+#  IN: 1 object
+#      2 datestamp
+# OUT: 1 reference to list with message-ID's
+
+sub _message_ids {
+
+# Obtain the object
+# Obtain the datestamp
+# Return now if nothing to do
+
+  my $self = shift;
+  my $datestamp = shift || '';
+  return [] unless $datestamp;
+
+# Obtain the work directory, making sure we have all the files of that date
+# Initialize the list of Message-ID's
+# For all of the files of that date
+#  Attempt to open the file
+#  Reloop if failed
+
+  my $work = _splat_datestamp( $self,$datestamp );
+  my @id;
+  foreach my $filename (sort {$a <=> $b} <$work/*>) {
+    my $handle = $self->openfile( $filename,'<' );
+    next unless $handle;
+
+#  While there are lines to be read
+#   Reloop if not a message-ID line
+#   Save the message-ID in the list and outloop
+#  Close the handle
+
+    while (<$handle>) {
+      next unless m#^message-id:\s*<?(.*?)>?$#i;
+      push( @id,$1 );
+      last;
+    }
+    close( $handle );
+  }
+
+# Return a reference to the list of message-IDs
+
+  return \@id;
+} #_message_ids
+
+#------------------------------------------------------------------------
+
+#  IN: 1 object
+#      2 Net::NNTP object
+#      3 name of newsgroup (default: name of targz)
+# OUT: 1 ordinal number of first message to fetch
+#      2 ordinal number of last message to fetch
+
+sub _resync_news {
+
+# Obtain the object
+# Obtain the Net::NNTP object
+# Obtain the newsgroup name
+
+  my $self = shift;
+  my $nntp = shift;
+  my $newsgroup = shift || $self->name;
+
+# Obtain the number of articles and the ordinal numbers
+# Return now if nothing to be done
+
+  my ($articles,$first,$last) = $nntp->group( $newsgroup );
+  return (0,-1) unless $articles || '';
+
+# Initialize the hash of headers fetched
+# Initialize the hash of associated times
+# Obtain the available datestamps
+# Remember the last one
+
+  my %head;
+  my %time;
+  my @datestamp = @{$self->datestamps};
+  my $lastdate = $datestamp[-1];
+
+# If successful in obtaining the header of the first message
+#  If successful in obtaining the time from that header
+#   Return now if that time is after what we have (a hole in the message stream)
+
+  if (my $head = $head{$first} ||= $nntp->head( $first )) {
+    if (my $time = $time{$first} ||= _epoch( $head )) {
+      return ($first,$last) if _datestamp( $time ) > $lastdate;
+    }
+  }
+
+# Initialize the datestamp
+# Initialize the message to start checking
+# If we can obtain a header of this message
+#  If we can obtain a time for this header
+#   Convert to datestamp
+
+  my $datestamp = 0;
+  my $check = $last;
+  if (my $head = $head{$check} ||= $nntp->head( $check )) {
+    if (my $time = $time{$check} ||= _epoch( $head )) {
+      $datestamp = _datestamp( $time );
+    }
+  }
+
+# If we're not on the right date yet
+#  Initialize the message number that was last checked
+#  Initialize the initial number that will be checked
+
+  if ($datestamp != $lastdate) {
+    my $lastchecked = 0;
+    $check = int(($last+$first)/2);
+
+#  Initialize the low mark
+#  Initialize the high mark
+#  While we still need to continue checking
+#   Remember which one we're checking now
+
+    my $low = $first;
+    my $high = $last;
+    while ($check != $lastchecked and $check <= $last) {
+      $lastchecked = $check;
+
+#   If successful in obtaining a reference to the header of that message
+#    If successful in obtaining the time of the message
+#     Convert to datestamp
+#     Outloop if we're on the right date
+#     If too low now
+#      Move to half of upper half and set new low mark
+#     Elseif too high now
+#      Move to lower half and set new high mark
+#     Reloop
+#   Move to the next message (no message with this number)
+
+      if (my $head = $head{$check} ||= $nntp->head( $check )) {
+        if (my $time = $time{$check} ||= _epoch( $head )) {
+          $datestamp = _datestamp( $time );
+          last if $datestamp == $lastdate;
+          if ($datestamp < $lastdate) {
+            ($low,$check) = ($check,int(($high+$check)/2));
+          } elsif ($datestamp > $lastdate) {
+            ($high,$check) = ($check,int(($check+$low)/2));
+          }
+          next;
+        }
+      }
+      $check++;
+    }
+  }
+
+# If we're on the right date now
+#  While there are messages to check
+#   If successful in obtaining the header of the next message
+#    If successful in obtaining a time from that
+#     Outloop if we're now on a different date
+#   Increment checked message number
+
+  if ($datestamp == $lastdate) {
+    while ($check < $last) {
+      if (my $head = $head{$check+1} ||= $nntp->head( $check+1 )) {
+        if (my $time = $time{$check+1} ||= _epoch( $head )) {
+          last if _datestamp( $time ) != $lastdate;
+        }
+      }
+      $check++;
+    }
+  }
+
+# Initialize the list with message id's
+# While there are still messages to check
+#  If successful in obtaining the header of this message
+#   If successful in obtaining the time of this message
+#    If there is a message-id in this message
+#     Obtain the datestamp of the time
+#     Obtain the message-id:s that we have for this datestamp
+#     For all of the ID's of this datestamp
+#      Return starting from the next if we know about this message-id
+#  Move to the previous message
+
+  my %ids;
+  while ($check >= $first) {
+    if (my $head = $head{$check} ||= $nntp->head( $check )) {
+      if (my $time = $time{$check} ||= _epoch( $head )) {
+        if (my ($id) = grep( s#^message-id:\s*<?(.*?)>?\n$#$1#si,@{$head} )) {
+          my $datestamp = _datestamp( $time );
+          my @id = @{$ids{$datestamp} ||= _message_ids( $self,$datestamp )};
+          foreach (@id) {
+            return ($check+1,$last) if $_ eq $id;
+          }
+        }
+      }
+    }
+    $check--;
+  }
+
+# Return the result, do them all
+
+  return ($first,$last);
+} #_resync_news
+
+#------------------------------------------------------------------------
+
 #  IN: 1 object
 #      2 datestamp to splat
 #      3 extension (including period) to use
@@ -1031,31 +1473,19 @@ sub _splat_datestamp {
     return $work;
   }
 
-# Obtain the current directory
 # Attempt to create that directory
-# If moving to the subdirectory successful
-#  Obtain the directory of the targz
-#  Obtain its name
-#  Obtain the subdirectory type
+# If the subdirectory exists now
 #  Extract all the files from that directory
 # Else (could not chdir to the directory)
 #  Add error
+# Return the directory to which we extracted
 
-  my $cwd = cwd;
   mkdir( $work,0777 );
-  if (chdir( $work )) {
-    my $directory = $self->directory;
-    (my $name = $directory) =~ s#^.*/##;
-    my $type = $extension =~ m#(\w+)$# ? $1 : 'rfc';
-    _extract_tarfile( $self,"$directory/$type/$datestamp.$name.tar.gz" );
+  if (-d $work) {
+    _extract_tarfile( $self,$self->tarfile( $datestamp,$extension ),$work );
   } else {
-    $self->_add_error( "Cannot chdir to '$work': $!" );
+    $self->_add_error( "Directory '$work' does not exist" );
   }
-
-# Restore the original directory
-# Return the directory
-
-  chdir( $cwd );
   return $work;
 } #_splat_datestamp
 
@@ -1073,11 +1503,20 @@ sub _str2time {
   my $time;
 
 # Return now if we found a time
+# Perform some heuristics to often occurring errors in date strings
+
+  return $time if $time = Date::Parse::str2time( $date );
+  $date =~ s#^(\w+,)(\w)#$1 $2#;
+  $date =~ s#(\d+)(\+\d+)$#$1 $2#;
+
+# Return now if we found a time this time
 # Return with nothing if there is no timezone specification at the end
+# Return now if there is nothing left to check
 # Do it again without the timezone spec and return the result
 
   return $time if $time = Date::Parse::str2time( $date );
   return '' unless $date =~ s#\s+[a-zA-Z]+$##;
+  return '' unless $date;
   return Date::Parse::str2time( $date ) || '';
 } #_str2time
 
@@ -1171,13 +1610,13 @@ messages and being able to access them quickly.  Older versions of this
 software that were used internally at NexTrieve, used a monthly tarfile for
 low-volume newsgroups.  But since the invention of super-economic file-systems
 such as ReiserFS (which is B<highly recommended> as the file system of choice
-for storing Targz archives on), the overhead of having a single tar-file per
+for storing Targz archives on), the overhead of having a single tarfile per
 date seems to be more than bearable, especially compared to the simplicity it
 creates.
 
-Apart from the tar-files that are created, no external files are being kept.
+Apart from the tarfiles that are created, no external files are being kept.
 Ordinal numbers for messages are determined by the files that are already in
-a tar-file and nothing else.
+a tarfile and nothing else.
 
 Internally at NexTrieve, this archive format is in use for storing over
 65 million messages from over 10000 textual(non-binary) newsgroups in about
@@ -1190,13 +1629,14 @@ HTML on a web-server.
 Currently a "tar" program that understands the following parameters B<must>
 be available for this module to operate correctly:
 
- --create        create new tar-file
+ --create        create new tarfile
+ --directory=    extract files to indicated directory
  --extract       extract files from archive
- --file=         specify name of tar-file to work on
- --gzip          filter tar-file through "gzip"
- --gunzip        filter tar-file through "gunzip"
- --list          list filenames of files stored in tar-file
- --remove-files  remove original files upon storing in tar-file
+ --file=         specify name of tarfile to work on
+ --gzip          filter tarfile through "gzip"
+ --gunzip        filter tarfile through "gunzip"
+ --list          list filenames of files stored in tarfile
+ --remove-files  remove original files upon storing in tarfile
  --to-stdout     extract files to STDOUT
  --verbose       perform action verbosely (list files being extracted)
 
@@ -1275,18 +1715,64 @@ in one or more Unix mailboxes to the targz.  Returns true if successful.  If
 the L<rm_original> method was previously called with a true value, then the
 files specified will be deleted on successful execution of this method.
 
+=head2 add_news
+
+  $targz->add_news( $nntp ) || die "could not add news\n";;
+
+The "add_news" method allows you to add RFC822 messages from a news server.
+The input parameter specifies the Net::NNTP object that should be used to
+obtain messages from the newsgroup of the L<name> of the targz.  Returns the
+number of messages that were successfully obtained.
+
+=head2 clean
+
+ $exit = $targz->clean;
+
+The "clean" method cleans the temporary directory in use by the object.  It
+is usually called automatically when the object is DESTROYed, unless inhibited
+by a call to the L<no_auto_clean> method.
+
+It returns the exit status of the system's "rm" command.
+
 =head2 count
 
  $all = $targz->count;
  $some = $targz->count( 'regexp' );
  $oneday = $targz->count( datestamp );
+ $stored = $targz->count( '',$hashref );
 
-The "count" method returns the number in the targz.  The amount can be for the
-whole targz or constraint by a regular expression (as used in a "grep()") or
-for just a single date (if a datestamp is specified).
+The "count" method returns the number of messages in the targz.  The amount
+can be for the whole targz or constraint by a regular expression (as used in
+a "grep()") or for just a single date (if a datestamp is specified).
+
+The optional second input parameter is a reference to a hash.  This hash will
+be filled with a key for each datestamp for which files are found.  The value
+of the key in the hash is a reference to a list which currently contains two
+value: the last modified time of the tarfile and the number of messages in it.
+If the tarfile is deemed to not have changed, the tarfile itself will not be
+read but instead the value found in the value will be used.  The hash reference
+can e.g. be stored in the directory with the Storable module, which is what the
+L<count_storable> method does, or can be stored in any other database backend
+that you might desire.
 
 Use the L<ids> method to find out the actual ID's of the messages.
 
+=head2 count_storable
+
+ $all = $targz->count_storable;
+ $some = $targz->count_storable( 'regexp' );
+ $oneday = $targz->count_storable( datestamp );
+
+The "count_storable" method is similar to the L<count> method, but it uses a
+hash that is stored in an external file ("count.gz" in the L<directory>
+directory) to remember which tarfiles were counted already before.  If the
+Storable module is not available, calling this method will still work but
+the counting will be much slower.
+
+The "count_storable" method returns the number of messages in the targz.  The
+amount can be for the whole targz or constraint by a regular expression (as
+used in a "grep()") or for just a single date (if a datestamp is specified).
+
 =head2 datestamps
 
  foreach (@{$targz->datestamps}) {
@@ -1334,15 +1820,49 @@ used in a "grep()") or for just a single date (if a datestamp is specified).
 Use the L<filename> method to find out the absolute filename of a message to
 be able to get at its contents.
 
+=head2 name
+
+  $name = $targz->name;
+
+The "name" method returns the name of the targz.  This is the same as the
+name of the final subdirectory on which the targz works.
+
+=head2 no_auto_clean
+
+ $targz->no_auto_clean( 1 );
+ $no_auto_clean = $targz->no_auto_clean;
+
+The "no_auto_clean" method specifies whether the temporary directory that is
+used by the object should be L<clean>ed when the object is DESTROYed.  By
+default, the object cleans the temporary directory.  A true value indicates
+that the temporary directory should B<not> be removed when the object is
+DESTROYed.  This is generally only useful in debugging situations.
+
 =head2 rm_original
 
  $targz->rm_original( 1 );
  $rm_original = $targz->rm_original;
 
-The "rm_original" specifies whether files that are specified to be added
+The "rm_original" method specifies whether files that are specified to be added
 (with either the L<add_file> or L<add_mbox> method) are automatically removed
 from the file system upon successful adding.
 
+=head2 tarfile
+
+ $tarfile = $targz->tarfile( '20020323' );
+ $tarfile = $targz->tarfile( $datestamp,'xml' );
+
+The "tarfile" method returns the absolute name of the tarfile that contains
+the files of a given date.  It is only necessary if you want to do some
+low level action on the tarfile.
+
+The first input parameter specifies the datestamp of the date of which you
+want to know the tarfile name.
+
+The optional second input parameter specifies which type of information you
+want the tarfile name of.  Two values are currently supported: 'rfc' and 'xml'.
+The value 'rfc' will be assumed if this input parameter is not specified.
+
 =head2 update_xml
 
  $files = $targz->updatexml( | $rfc822, | $docseq );
